      SUBROUTINE  UNTAB (LINE,MAXLIN,WORK,LENGTH)
C     (REMOVE TABS)
C     THIS ROUTINE PROVIDES THE SERVICE OF TAB CHARACTER REMOVAL
C     FOR UTILITIES WHICH PROCESS FORTRAN PROGRAMS.  THE DEC-20
C     COMPILER PLACES THE FOLLOWING INTERPRETATION ON TABS:
C
C     * AN INITIAL TAB FOLLOWED BY A DIGIT 1..9 PLACES THE
C       DIGIT IN COLUMN 6 AND THE FOLLOWING TEXT IN COLUMN 7.
C
C     * AN INITIAL TAB NOT FOLLOWED BY A DIGIT 1..9 PLACES THE
C       FOLLOWING TEXT IN COLUMN 7.
C
C     * NON-INITIAL TABS POSITION TO THE NEXT COLUMN WHICH IS
C       A MULTIPLE OF 8, AND ARE REPLACED BY ONE OR MORE BLANKS.
C
C     INITIAL LINE NUMBERS CANNOT BE DETECTED, BECAUSE FORTRAN
C     REMOVES THE BIT WHICH FLAGS THEM.  INPUT FILES MUST THEREFORE
C     NOT CONTAIN LINE NUMBERS.  NOTE THAT TAB CHARACTERS EMBEDDED
C     IN STRING CONSTANTS WILL ALSO BE EXPANDED TO BLANKS, THUS
C     DEVIATING FROM FORTRAN'S PRACTICE OF LEAVING THEM INTACT INSIDE
C     SUCH CONSTANTS.
C
C     THE ARGUMENTS ARE:
C     LINE(*)...........LINE CONTAINING PACKED CHARACTERS.  ON RETURN,
C                       IT WILL CONTAIN THE EXPANDED TEXT WITH ALL
C                       TABS REMOVED.
C     MAXLIN............MAXIMUM NUMBER OF CHARACTERS WHICH CAN BE
C                       STORED IN LINE(*).  CHARACTERS EXTENDING
C                       BEYOND THIS LIMIT BECAUSE OF TAB EXPANSION
C                       WILL BE DISCARDED.
C     WORK(*)...........WORK ARRAY AS LARGE AS LINE(*).
C     LENGTH............NUMBER OF CHARACTERS PACKED IN LINE(*).
C                       IT WILL BE UPDATED ON RETURN.
C
C     THIS SERVICE CAN BE RATHER COSTLY.  ON A TEST ON THE
C     DEC-20/60, 5000 80-CHARACTER LINES WERE READ IN 7.8 SEC;
C     WITH 5000 CALLS TO UNTAB ADDED, THE TIME JUMPED TO 27.8
C     SEC, AN INCREASE BY A FACTOR OF 3.6.
C     (17-AUG-80)
      INTEGER          CHAR,        HT,          KARASC,      LENGTH
      INTEGER          LENOLD,      LIMIT,       LINE(1),     MAXLIN
      INTEGER          MIN0,        NEXTCH,      WORK(1)
      DATA HT/9/
      CALL KARMOV (WORK,1,LINE,1,LENGTH)
      LENOLD = LENGTH
C
C     SPECIAL HANDLING FOR INITIAL CHARACTER
C
      CALL KARGET (CHAR,WORK,1)
      CHAR = KARASC(CHAR)
      IF (.NOT.(CHAR .EQ. HT)) GO TO 20001
      CALL KARGET (CHAR,WORK,2)
      CHAR = KARASC(CHAR)
      IF (.NOT.((49 .LE. CHAR) .AND. (CHAR .LE. 57))) GO TO 20004
      NEXTCH = 3
      LENGTH = 6
      CALL KARMOV (LINE,1,6H     #,1,6)
      GO TO 20005
20004 NEXTCH = 2
      LENGTH = 6
      CALL KARMOV (LINE,1,6H      ,1,6)
20005 CONTINUE
20006 GO TO 20002
20001 LENGTH = 0
      NEXTCH = 1
20002 CONTINUE
C
C     COPY REMAINING CHARACTERS, EXPANDING TABS AS REQUIRED.
C
20003 CONTINUE
20007 IF (.NOT.((LENGTH .LT. MAXLIN) .AND. (NEXTCH .LE. LENOLD))) GO TO
     X20008
      CALL KARGET (CHAR,WORK,NEXTCH)
      IF (.NOT.(KARASC(CHAR) .EQ. HT)) GO TO 20009
      LIMIT = MIN0(8*((LENGTH+8)/8),MAXLIN)
20012 IF (.NOT.(LENGTH .LT. LIMIT)) GO TO 20013
      LENGTH = LENGTH + 1
      CALL KARPUT (1H ,LINE,LENGTH)
      GO TO 20012
20013 GO TO 20010
20009 LENGTH = LENGTH + 1
      CALL KARPUT (CHAR,LINE,LENGTH)
20010 NEXTCH = NEXTCH + 1
      GO TO 20007
20008 RETURN
      END
